<?php
/*
 * About WebsiteBaker
 *
 * Website Baker is a PHP-based Content Management System (CMS)
 * designed with one goal in mind: to enable its users to produce websites
 * with ease.
 *
 * LICENSE INFORMATION
 *
 * WebsiteBaker is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * WebsiteBaker is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
/**
 *
 * @category        core
 * @package         framework
 * @subpackage      frontend
 * @copyright       WebsiteBaker Org. e.V.
 * @author          Dietmar Wöllbrink <dietmar.woellbrink@websitebaker.org>
 * @author          Manuela v.d.Decken <manuela@isteam.de>
 * @link            http://websitebaker.org/
 * @license         GNU General Public License 2.0
 * @platform        WebsiteBaker 2.11.0
 * @requirements    PHP 5.6.3 and higher
 * @version         $Id: frontend.functions.php 151 2018-10-09 11:54:18Z Luisehahne $
 * @since           File available since 18.10.2017
 * @deprecated      no
 * @description     This file is where some of the WB frontend functions are stored.
 *
 */
//declare(strict_types = 1);
//declare(encoding = 'UTF-8');
/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
    if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 File not found'; flush(); exit;}
/* -------------------------------------------------------- */
    if (!isset($wb) || !($wb instanceof frontend)) { $wb = new \frontend(); }
// deprecated only backwardscompability because modules are using backend object $admin in the frontend
    if (isset($wb) && !isset($admin)) {$admin = $wb; }
    $page_id = (!isset($page_id) ? $wb->default_page_id : $page_id);

    if (!registerSnippets($page_id)){/* nothing to do only for loading the snippets include function */}
// Frontend functions
    if (!\function_exists('page_link')) {
        /**
         * generate full qualified URL from relative link based on pages_dir
         * @param string $link
         * @return string
         */
        function page_link($link) {
            return $GLOBALS['wb']->page_link($link);
        }
    }

    if (!function_exists('get_page_link')) {
        /**
         * get relative link from database based on pages_dir
         * @global <type> $database
         * @param <type> $id
         * @return <type>
         */
        function get_page_link( $id )
        {
            global $database;
            $sql = 'SELECT `link` FROM `'.TABLE_PREFIX.'pages` '
                 . 'WHERE `page_id` = '.\intval($id);
            $link = $database->get_one( $sql );
            return $link;
        }
    }

//function to highlight search results
    if (!function_exists('search_highlight')) {
        /**
         *
         * @staticvar boolean $string_ul_umlaut
         * @staticvar boolean $string_ul_regex
         * @param string $foo
         * @param array $arr_string
         * @return string
         */
        function search_highlight($foo='', $arr_string=array()) {
            require_once(WB_PATH.'/framework/functions.php');
            static $string_ul_umlaut = FALSE;
            static $string_ul_regex = FALSE;
            if($string_ul_umlaut === FALSE || $string_ul_regex === FALSE) {
                require(WB_PATH.'/search/search_convert.php');
            }
            $foo = entities_to_umlauts($foo, 'UTF-8');
            \array_walk($arr_string, function(&$val,$key) {$val = \preg_quote($val, '~');});
            $search_string = \implode("|", $arr_string);
            $string = \str_replace($string_ul_umlaut, $string_ul_regex, $search_string);
            // the highlighting
            // match $string, but not inside <style>...</style>, <script>...</script>, <!--...--> or HTML-Tags
            // Also droplet tags are now excluded from highlighting.
            // split $string into pieces - "cut away" styles, scripts, comments, HTML-tags and eMail-addresses
            // we have to cut <pre> and <code> as well.
            // for HTML-Tags use <(?:[^<]|<.*>)*> which will match strings like <input ... value="<b>value</b>" >
            $matches = \preg_split("~(\[\[.*\]\]|<style.*</style>|<script.*</script>|<pre.*</pre>|<code.*</code>|<!--.*-->|<(?:[^<]|<.*>)*>|\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,8}\b)~iUs",$foo,-1,(PREG_SPLIT_DELIM_CAPTURE|PREG_SPLIT_NO_EMPTY));
            if (\is_array($matches) && $matches != array()) {
                $foo = "";
                foreach($matches as $match) {
                    if($match{0}!="<" && !\preg_match('/^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,8}$/i', $match) && !preg_match('~\[\[.*\]\]~', $match)) {
                        $match = \str_replace(array('&lt;', '&gt;', '&amp;', '&quot;', '&#039;', '&nbsp;'), array('<', '>', '&', '"', '\'', "\xC2\xA0"), $match);
                        $match = \preg_replace('~('.$string.')~ui', '_span class=_highlight__$1_/span_',$match);
                        $match = \str_replace(array('&', '<', '>', '"', '\'', "\xC2\xA0"), array('&amp;', '&lt;', '&gt;', '&quot;', '&#039;', '&nbsp;'), $match);
                        $match = \str_replace(array('_span class=_highlight__', '_/span_'), array('<span class="highlight">', '</span>'), $match);
                    }
                    $foo .= $match;
                }
            }

            if(DEFAULT_CHARSET != 'utf-8') {
                $foo = umlauts_to_entities($foo, 'UTF-8');
            }
            return $foo;
        }
    }

    if (!function_exists('page_content')) {
        /**
         *
         * @global array $TEXT
         * @global array $MENU
         * @global array $HEADING
         * @global array $MESSAGE
         * @global array $globals several global vars
         * @global datadase $database
         * @global wb $wb
         * @global string $global_name
         * @param int $block
         * @return void
         */
        function page_content($block = 1) {
            // Get outside objects
            global $TEXT,$MENU,$HEADING,$MESSAGE;
            global $globals;
            global $database;
            global $wb;
            $admin = $wb;

            if ($wb->page_access_denied==true) {
                echo $MESSAGE['FRONTEND_SORRY_NO_VIEWING_PERMISSIONS']; //  $aMsg[] =
                return;
            }

            if ($wb->page_no_active_sections==true) {
                echo $MESSAGE['FRONTEND_SORRY_NO_ACTIVE_SECTIONS'];
                return;
            }

            if(isset($globals) AND \is_array($globals)) {
                foreach($globals AS $global_name) {
                    global $$global_name;
                }
            }
            // Make sure block is numeric
            if (($block = \intval($block)) == 0 ) { $block = 1;}

            // Include page content
            if (!\defined('PAGE_CONTENT') OR $block!=1)
            {
                $page_id = \intval($wb->page_id);
                $iNow = \time();
            // First get all sections for this page   , `publ_start`, `publ_end`
                $sqlSet = 'SELECT `section_id`, `module` '
                        . 'FROM `'.TABLE_PREFIX.'sections` '
                        . 'WHERE `page_id`='.$page_id.' AND `block`='.$block.' AND'
                         .       '('.$iNow.' BETWEEN `publ_start` AND `publ_end`) OR '
                         .       '('.$iNow.' > `publ_start` AND `publ_end`=0) '
                        . 'ORDER BY `position`';
                if (!($query_sections = $database->query($sqlSet)) ) { return; }
            // If none were found, check if default content is supposed to be shown
                if ($query_sections->numRows() == 0) {
                    if ($wb->default_block_content == 'none') { return; }
                    if (\is_numeric($wb->default_block_content)) {
                        $page_id = $wb->default_block_content;
                    } else {
                        $page_id = $wb->default_page_id;
                    }
                    if (!($query_sections = $database->query($sqlSet))) {return;}
                    // Still no cotent found? Give it up, there's just nothing to show!
    //                if($query_sections->numRows() == 0) { return; }
                }
                // Loop through them and include their module file
                while($section = $query_sections->fetchRow(MYSQLI_ASSOC)) {
                    // skip this section if it is out of publication-date
                    $section_id = $section['section_id'] ;
                    $module = $section['module'] ;
                    // check if module exists - feature: write in errorlog
                    if (\is_readable(WB_PATH.'/modules/'.$module.'/view.php')) {
                     // fetch content -- this is where to place possible output-filters (before highlighting)
                     \ob_start() ; // fetch original content<div id="Sec103" class="section  m_modulename user-defined-class" >
                     $sSectionIdPrefix = (\defined('SEC_ANCHOR') && SEC_ANCHOR != '') ? SEC_ANCHOR : '' ;

                     require (WB_PATH.'/modules/'.$module.'/view.php') ;
                     $content = \ob_get_clean() ;
                     if($content != '') {

                          $sAnchor = "\n".'<div id="'.$sSectionIdPrefix.$section_id.'" class="section m_'.$module.'" >';
                          $sBeforeContent = ($sSectionIdPrefix == 'none') ? '' : $sAnchor."\n";
                          $sAfterContent  = ($sSectionIdPrefix == 'none') ? '' : "\n".'</div><!-- '.$module.$section_id.' -->'."\n" ;
                          $content = $sBeforeContent.$content.$sAfterContent ;
                     }
                    } else {
                        continue;
                    }
                    // highlights searchresults
                    if (isset($_GET['searchresult']) && \is_numeric($_GET['searchresult'])
                              && !isset($_GET['nohighlight'])
                              && isset($_GET['sstring'])
                              && !empty($_GET['sstring'])
                              ) {
                        $arr_string = \explode(" ", $_GET['sstring']);
                        if($_GET['searchresult']==2) { // exact match
                            $arr_string[0] = \str_replace("_", " ", $arr_string[0]);
                        }
                        echo search_highlight($content, $arr_string);
                    } else {
                        echo $content;
                    }
                } // end while

            } else {   // Searchresults! But also some special pages,
             // e.g. guestbook (add entry), news (add comment) uses this
                \ob_start(); // fetch original content
                require(PAGE_CONTENT);
                $content = ob_get_clean();
                // Apply Filters
                if (\function_exists('OutputFilterApi')) {
                    $content = OutputFilterApi('OpF?arg=special', $content);
                }
                echo $content;
            }
        }
    }

    if (!function_exists('show_content')) {
        function show_content($block=1) {
            page_content($block);
        }
    }

    if (!function_exists('show_breadcrumbs'))
    {
        function show_breadcrumbs($sep = ' &raquo; ',$level = 0, $links = true, $depth = -1, $title = '', $print=true)
        {
            global $wb,$database;
            $oLang = Translate::getInstance();
            $oLang->enableAddon('templates\\'.DEFAULT_TEMPLATE);
            $retVal = '';
            $page_id = $wb->page_id;
            $title = (\trim($title) == '') ? $oLang->MENU_BREADCRUMB : $title;
            if ($page_id != 0)
            {
                $counter = 0;
                // get links as array
                $bread_crumbs = $wb->page_trail;
                $count = \sizeof($bread_crumbs);
                // level can't be greater than sum of links
                $level = ($count <= $level ) ? $count-1 : $level;
                // set level from which to show, delete indexes in array
                $crumbs = \array_slice($bread_crumbs, $level );
                $depth = ($depth <= 0) ? \sizeof($crumbs) : $depth;
                // if empty array, set orginal links
                $crumbs = (!empty($crumbs)) ?  $crumbs : $wb->page_trail;
                $total_crumbs = ( ($depth <= 0) || ($depth > \sizeof($crumbs)) ) ? \sizeof($crumbs) : $depth;
                $retVal .= '<div class="breadcrumb">'.PHP_EOL.'<span class="title">'.$title.'</span>'.PHP_EOL;
                foreach ($crumbs as $temp)
                {
                    if($counter == $depth) { break; }
                    // set links and separator
                        $sql  = 'SELECT * FROM `'.TABLE_PREFIX.'pages` WHERE `page_id`='.(int)$temp;
                        $query_menu = $database->query($sql);
                        $page = $query_menu->fetchRow();
                        $show_crumb = (($links == true) && ($temp != $page_id))
                                ? '<a href="'.page_link($page['link']).'" class="link">'.$page['menu_title'].'</a>'.PHP_EOL
                                : '<span class="crumb">'.$page['menu_title'].'</span>'.PHP_EOL;
                        // Permission
                        switch ($page['visibility'])
                        {
                            case 'none' :
                            case 'hidden' :
                            // if show, you know there is an error in a hidden page
                                $retVal .= $show_crumb.'&nbsp;';
                                break;
                            default :
                                $retVal .= $show_crumb;
                                break;
                        }
                        if ( ( $counter <> $total_crumbs-1 ) )
                        {
                            $retVal .= '<span class="separator">'.$sep.'</span>'.PHP_EOL;
                        }
                    $counter++;
                }
                $retVal .=  "</div>".PHP_EOL;
                if ($print) { print $retVal;} else {return $retVal;}
            }
        }
    }

// Function for page title
    if (!\function_exists('page_title')) {
        function page_title($spacer = ' - ', $template = '[WEBSITE_TITLE][SPACER][PAGE_TITLE]') {
            $vars = array('[WEBSITE_TITLE]', '[PAGE_TITLE]', '[MENU_TITLE]', '[SPACER]');
            $sWebsiteTitle = (\defined('WEBSITE_TITLE') ? WEBSITE_TITLE : '');
            $sPageTitle    = (\defined('PAGE_TITLE') ? PAGE_TITLE : '');
            $sMenuTitle    = (\defined('MENU_TITLE') ? MENU_TITLE : '');
            $values = array($sWebsiteTitle, $sPageTitle, $sMenuTitle, $spacer);
            echo \str_replace($vars, $values, $template);
        }
    }

// Function for page description
    if (!\function_exists('page_description')) {
        function page_description() {
            global $wb;
            if ($wb->page_description!='') {
                echo $wb->page_description;
            } else {
                echo WEBSITE_DESCRIPTION;
            }
        }
    }

// Function for page keywords
    if (!\function_exists('page_keywords')) {
        function page_keywords() {
            global $wb;
            if ($wb->page_keywords!='') {
                echo $wb->page_keywords;
            } else {
                echo WEBSITE_KEYWORDS;
            }
        }
    }

// Function for page header
    if (!\function_exists('page_header')) {
        function page_header($date_format = 'Y') {
            echo WEBSITE_HEADER;
        }
    }

// Function for page footer
    if (!\function_exists('page_footer')) {
        function page_footer($date_format = 'Y') {
            global $starttime;
            $vars = array('[YEAR]', '[PROCESS_TIME]');
            $processtime=\array_sum(\explode(" ",\microtime()))-$starttime;
            $values = array(\gmdate($date_format),$processtime);
            echo \str_replace($vars, $values, WEBSITE_FOOTER);
        }
    }

/* ------------------------------------------------------------------------------------------------ */
//  begin register_frontend_files
/* ------------------------------------------------------------------------------------------------ */
    function registerSnippet(){
        global $database;
        $aSnippetsRec = [];
        $aAlllowedModules = [];
        $aModules = \glob (WB_PATH.'/modules/*', \GLOB_ONLYDIR|\GLOB_NOSORT);
        foreach ($aModules as $sPath){
            if (\is_readable($sPath.'/include.php')){$aAlllowedModules[] = \basename($sPath);}
        }
        $sAllowedModules   = \implode(', ',
                             \array_map(function(&$item) use ($database){
                                 return '\''.$database->escapeString($item) .'\'';
                             },$aAlllowedModules));

        $sql  = 'SELECT `directory`,`function` FROM `'.TABLE_PREFIX.'addons` '
              . 'WHERE `type`=\'module\' '
              .   'AND `function` IN (\'snippet\', \'tool\') '
             .    'AND `directory` IN('.$sAllowedModules.')'
              . '';
        if (($oSnippets = $database->query($sql)))
        {
            $aSnippetsRec = $oSnippets->fetchAll( MYSQLI_ASSOC);
        } else {
            \trigger_error ($database->get_error(),E_USER_WARNING);
        }
        return $aSnippetsRec;
    }

    function register_frontend_SnippetCss($page_id){
        static $bLoaded   = false;
        $aRetval = [];
        if (!$bLoaded) {
            $bLoaded = true;
        }
            $aFrontend = registerSnippet();
            foreach($aFrontend as $aValue) {
                $sAddonFile = '/modules/'.$aValue['directory'].'/frontend.css';
                $sAddonFile = (\is_readable(WB_PATH.'/modules/'.$aValue['directory'].'/frontendUser.css')
                              ? '/modules/'.$aValue['directory'].'/frontendUser.css' : $sAddonFile);
                if (\is_readable(WB_PATH.$sAddonFile)){
                    $aRetval[] = $sAddonFile;
                }
            }
        return $aRetval;
    }

    function register_frontend_SnippetJs($page_id){
        static $bLoaded = false;
        $aRetval = [];
        if (!$bLoaded) {
            $bLoaded = true;
        }
            $aFrontend = registerSnippet();
            foreach($aFrontend as $aValue) {
                $sAddonFile = '/modules/'.$aValue['directory'].'/frontend.js';
                $sAddonFile = (\is_readable(WB_PATH.'/modules/'.$aValue['directory'].'/frontendUser.js')
                              ? '/modules/'.$aValue['directory'].'/frontendUser.js' : $sAddonFile);
                if (\is_readable(WB_PATH.$sAddonFile.'')){
                    $aRetval[] = $sAddonFile;
                }
            }
        return $aRetval;
    }

    function register_frontend_SnippetBodyJs($page_id){
        static $bLoaded = false;
        $aRetval = [];
        if (!$bLoaded) {
            $bLoaded = true;
        }
            $aFrontend = registerSnippet();
            foreach($aFrontend as $aValue) {
                $sAddonFile = '/modules/'.$aValue['directory'].'/frontend_body.js';
                if (\is_readable(WB_PATH.$sAddonFile.'')){
                    $aRetval[] = $sAddonFile;
                }
            }
        return $aRetval;
    }

/* -------------------------------------------------------------------------------------- */
    function registerFrontend($page_id){
        global $database;
        $aAddonsRec = [];
        // workout to included frontend.css, fronten.js and frontend_body.js in pages
            $sql  = 'SELECT DISTINCTROW `s`.`module` `directory`, `a`.`function`  FROM `'.TABLE_PREFIX.'sections` `s` '
                  . 'INNER JOIN `'.TABLE_PREFIX.'addons` `a`  '
                  . ' ON `s`.`module` = `a`.`directory`'
                  . 'WHERE `s`.`page_id` = '.(int)$page_id.' '
              .   'AND `function` IN (\'page\') '
                  . '';
            if (($oAddons = $database->query($sql)))
            {
                $aAddonsRec = $oAddons->fetchAll(MYSQLI_ASSOC);
            } else {
                \trigger_error ($database->get_error(),E_USER_WARNING);
            }
        return $aAddonsRec;
    }

    function register_frontend_FrontendCss($page_id){
        static $bLoaded = false;
        $aRetval = [];
        if (!$bLoaded) {
            $aFrontend = registerFrontend($page_id);
            foreach($aFrontend as $aValue) {
                $sAddonFile = '/modules/'.$aValue['directory'].'/frontend.css';
                $sAddonFile = (\is_readable(WB_PATH.'/modules/'.$aValue['directory'].'/frontendUser.css')
                              ? '/modules/'.$aValue['directory'].'/frontendUser.css' : $sAddonFile);
                if (!\is_readable(WB_PATH.$sAddonFile)){continue;}
                $aRetval[] = $sAddonFile;
            }
            $bLoaded = true;
        }
        return $aRetval;
    }

    function register_frontend_FrontendJs($page_id){
        static $bLoaded = false;
        $aRetval = [];
        if (!$bLoaded) {
            $bLoaded = true;
        }
            $aFrontend = registerFrontend($page_id);
            foreach($aFrontend as $aValue) {
                $sAddonFile = '/modules/'.$aValue['directory'].'/frontend.js';
                $sAddonFile = (\is_readable(WB_PATH.'/modules/'.$aValue['directory'].'/frontendUser.js')
                              ? '/modules/'.$aValue['directory'].'/frontendUser.js' : $sAddonFile);
                if (\is_readable(WB_PATH.$sAddonFile.'')){
                    $aRetval[] = $sAddonFile;
                }
            }
        return $aRetval;
    }

    function register_frontend_FrontendBodyJs($page_id){
        static $bLoaded = false;
        $aRetval = [];
        if (!$bLoaded) {
            $bLoaded = true;
        }
            $aFrontend = registerFrontend($page_id);
            foreach($aFrontend as $aValue) {
                $sAddonFile = '/modules/'.$aValue['directory'].'/frontend_body.js';
                if (\is_readable(WB_PATH.$sAddonFile.'')){
                    $aRetval[] = $sAddonFile;
                }
            }
        return $aRetval;
    }

/* -------------------------------------------------------------------------------------- */
    function include_snippet($sModuleName=''){
        static $isIncluded = [];
        $aRetVal = [];
        if (!\in_array($sModuleName, $isIncluded))
        {
            if (\is_readable(WB_PATH.'/modules/'.$sModuleName.'/include.php'))
            {
                $sRetVal = '/modules/'.$sModuleName.'/include.php';
                include(WB_PATH.'/modules/'.$sModuleName.'/include.php');
                // check if already exists
                \array_push ($isIncluded, $sModuleName);
                $aRetVal = $isIncluded;
            }
        }
       return $aRetVal;
    }

/* -------------------------------------------------------- */
    function register_frontend_LoadOnFly ()
    {
        static $bSriptLoaded=false;
        $sLoadOnFly = [];
        if (!$bSriptLoaded){
            $sLoadOnFly[] = '/include/jquery/domReady-min.js';
            $sLoadOnFly[] = '/include/jquery/LoadOnFly-min.js';
            $bSriptLoaded = true;
        }
        return $sLoadOnFly;
    }

/* -------------------------------------------------------- */
    function register_frontend_ScriptVars ()
    {
        static $bSriptLoaded=false;
        $sScriptVars = [];
        if ($bSriptLoaded){ return $sScriptVars; }
            $sScriptVars[] = ''
                ."\n\t\t".'/* inserted by register_frontend_modfiles */'."\n"
                ."\t\t"."var URL = '".WB_URL."';\n"
                ."\t\t"."var WB_URL = '".WB_URL."';\n"
                ."\t\t"."var THEME_URL = '".THEME_URL."';\n"
                ."\t\t"."var TEMPLATE_DIR = '".TEMPLATE_DIR."';\n"
                ."\t\t"."var TEMPLATE = '".TEMPLATE."';\n"
                ."\t\t"."var EDITOR = '".WYSIWYG_EDITOR."';\n"
                ."\t\t"."var LANGUAGE = '".LANGUAGE."';\n"
                .'';
            $bSriptLoaded = true;
        return $sScriptVars;
    }

/* -------------------------------------------------------- */
function new_register_frontend_Jquery ()
{
    static $bLoaded = false;
    $jquery_links = [];
    // get it later by a register class
    $aVersion = ['core'=>'3.3.1', 'migrate'=>'1.4.1', 'ui'=>'1.9.2'];
    $aVersion = ['core'=>'1.12.41', 'migrate'=>'', 'ui'=>'1.8.17'];
    if (!$bLoaded) {
        /* include the Javascript jquery api  */
        if (\file_exists(WB_PATH .'/include/jquery/core/3.3.1/jquery-min.js'))
        {
            $aFilterSettings = getOutputFilterSettings();
            $bLoadJqUi = (isset($aFilterSettings['JqueryUI']) && $aFilterSettings['JqueryUI']);
            $jquery_links[] = '/include/jquery/core/'.$aVersion['core'].'/jquery-min.js';
            if ($aVersion['migrate']){
                $jquery_links[] = '/include/jquery/migrate/'.$aVersion['migrate'].'/jquery-migrate-min.js';
            }
            $jquery_links[] = '/include/jquery/plugins/jquery-insert.js';
            $jquery_links[] = '/include/jquery/plugins/jquery-include.js';
            /* workout to insert ui.css and theme */
            if ($bLoadJqUi){
                // only  load css
                $jquery_theme   =  '/modules/jquery/jquery_theme.js';
                $jquery_uitheme =  '/modules/jquery/ui/'.$aVersion['ui'].'/jquery-ui-min.js';
                $jquery_links[] =  (\file_exists(WB_PATH.$jquery_theme)
                    ? '/modules/jquery/jquery_theme.js'
                    : '/include/jquery/jquery_theme.js');
                $jquery_links[] =  (\file_exists(WB_PATH.$jquery_uitheme)
                    ? '/modules/jquery/ui/'.$aVersion['ui'].'/jquery-ui-min.js'
                    : '/include/jquery/ui/'.$aVersion['ui'].'/jquery-ui-min.js');
            }
        /* workout to insert plugins functions, set in templatedir */
            $jquery_frontend_file = \str_replace (WB_URL, '', TEMPLATE_DIR.'/jquery_frontend.js');
            $jquery_frontend_file = 'include/jquery/jquery_frontend.js';
            $jquery_frontend_file = '';
            $sTmp = (\is_readable(WB_PATH.'/'.$jquery_frontend_file)
                ? $jquery_frontend_file
                : '');
            if (!empty($sTmp)){
                $jquery_links[] = $sTmp;
            }
        $bLoaded = true;
        }
    }
    return $jquery_links;
}

/* -------------------------------------------------------- */
function register_frontend_Jquery ()
{
    static $bLoaded = false;
    $jquery_links = [];
    if (!$bLoaded) {
        $jquery_version = (\defined('JQUERY_VERSION')&&!empty(JQUERY_VERSION) ? JQUERY_VERSION : '1.12.4').'/';
        /* include the Javascript jquery api  */
        if (\file_exists(WB_PATH .'/include/jquery/dist/'.$jquery_version.'jquery-min.js'))
        {
            $aFilterSettings = getOutputFilterSettings();
            $bLoadJqUi = (isset($aFilterSettings['JqueryUI']) && $aFilterSettings['JqueryUI']);
            $jquery_links[] = '/include/jquery/dist/'.$jquery_version.'jquery-min.js';
            $jquery_links[] = '/include/jquery/dist/'.$jquery_version.'jquery-migrate-min.js';
            $jquery_links[] = '/include/jquery/jquery-insert.js';
            $jquery_links[] = '/include/jquery/jquery-include.js';
            /* workout to insert ui.css and theme */
            if ($bLoadJqUi){
                // only  load css
                $jquery_theme   =  '/modules/jquery/jquery_theme.js';
                $jquery_uitheme =  '/modules/jquery/jquery-ui-min.js';
                $jquery_links[] =  (\file_exists(WB_PATH.$jquery_theme)
                    ? '/modules/jquery/jquery_theme.js'
                    : '/include/jquery/jquery_theme.js');
                $jquery_links[] =  (\file_exists(WB_PATH.$jquery_uitheme)
                    ? '/modules/jquery/jquery-ui-min.js'
                    : '/include/jquery/jquery-ui-min.js');
            }
        /* workout to insert plugins functions, set in templatedir */
            $jquery_frontend_file = '';
            $jquery_frontend_file = '/include/jquery/jquery_frontend.js';
            $jquery_frontend_file = \str_replace (WB_URL, '', TEMPLATE_DIR.'/jquery_frontend.js');
            $sTmp = (\is_readable(WB_PATH.'/'.$jquery_frontend_file)
                ? $jquery_frontend_file
                : '');
            if (!empty($sTmp)){
                $jquery_links[] = $sTmp;
            }
        $bLoaded = true;
        }
    }
    return $jquery_links;
}

/* ------------------------------------------------------------------------------------- */
    // if you need to include snippets before calling by register_modfiles_frontend
    function registerSnippets($page_id){
        $aRetVal  = [];
        $aSnippets = registerSnippet($page_id);
/*
    $oRequest = \bin\requester\HttpRequester::getInstance();
    $oReg = \WbAdaptor::getInstance();
    $oReg->setDatabase(database::getInstance());
    $oApp = (isset($GLOBALS['admin'])&&($GLOBALS['admin'] instanceof admin) ? $GLOBALS['admin'] : $GLOBALS['wb']);
    $oReg->setApplication($oApp);
    $oReg->setRequester($oRequest);
    $oReg->setTranslate(\Translate::getInstance());
*/
        foreach($aSnippets as $aModuleDir) {
            $sModuleDir = $aModuleDir['directory'];
            $aRetVal = include_snippet($sModuleDir);
        }
        return $aRetVal;
    }
/*--------------------------------------------------------------------------------------*/
    function checkResult($mResults, $sFilter){
        $aRetVal[$sFilter] = [];
        $aTmp = (\is_array($mResults[$sFilter]) ? $mResults[$sFilter] : $aRetVal[$sFilter]);
        return $mResults;
    }

/* -------............................------------------------------------------------- */
    function register_addon_files(array $aFilter = [
                'SnippetCss',
                'FrontendCss',
                'ScriptVars',
                'LoadOnFly',
                'Jquery',
                'JqueryUI',
                'SnippetJs',
                'FrontendJs',
                'SnippetBodyJs',
                'FrontendBodyJs',
        ]){
        global $page_id;
        $aFrontendFiles  = [];

        $aSeekFilters = array_flip($aFilter);

        $aFilters = $aSeekFilters;
        $aFilters = array_intersect_key(getOutputFilterSettings(), $aSeekFilters);
        $aResults = [];
        foreach ($aFilters as $sName => $bValue) {
            $aResults[$sName] = [];
            if ($bValue) {
                $sFunction = 'register_frontend_'.$sName;
                if (function_exists($sFunction)) {
                    $aResults[$sName] = $sFunction($page_id);
                }
            }
        }

        foreach ($aSeekFilters as $sName=>$bValue) {
                if (isset($aResults[$sName])){
//                    echo $sName,' = ', (int)$aFilters[$sName].'<br />';  getResult($aResults, $sName)
                    switch ($sName)
                    {
                    case 'SnippetCss':
                    case 'FrontendCss':
                        $aResults = checkResult($aResults, $sName);
                        $aFrontendFiles[$sName] = $aResults[$sName];
                        break;
                    case 'ScriptVars':
                    case 'LoadOnFly':
                        $aResults = checkResult($aResults, $sName);
                        $aFrontendFiles[$sName] = $aResults[$sName];
                        break;
                    case 'Jquery':
                    case 'JqueryUI':
                        $aResults = checkResult($aResults, $sName);
                        $aFrontendFiles[$sName] = $aResults[$sName];
                        break;
                    case 'SnippetJs':
                    case 'FrontendJs':
                        $aResults = checkResult($aResults, $sName);
                        $aFrontendFiles[$sName] = $aResults[$sName];
                        break;
                    case 'SnippetBodyJs':
                    case 'FrontendBodyJs':
                        $aResults = checkResult($aResults, $sName);
                        $aFrontendFiles[$sName] = $aResults[$sName];
                        break;
                    }
                }
        } // end foreach
        return $aFrontendFiles;
    }
/* --------------------------------------------------------------------------------------*/

    function prepareLink($aOutput, $sFileType){
      $sRetval = '';
      if ($aOutput) {
          foreach($aOutput as $iKey => $sHeadLinks){
              if ($sHeadLinks!=''){
                  switch ($sFileType) {
                      case 'css':
                          $sRetval .= "\t".'<link rel="stylesheet" href="'.WB_URL.''.$sHeadLinks.'" media="screen" />'."\n";
                          break;
                      case 'js':
                          $sRetval .= "\t".'<script src="'.WB_URL.$sHeadLinks.'"></script>'."\n";
                          break;  //
                      case 'var':
                          $sRetval .= "\t".'<script>'.$sHeadLinks.'</script>'."\n";
                          break;  //
                      default:
                          break;  //
                  }  // end switch
              }
          }
      }
      return $sRetval;
    }
/* --------------------------------------------------------------------------------------*/

// Function to add optional module Javascript or CSS stylesheets into the <head> section of the frontend
    if (!function_exists('register_frontend_modfiles')) {
        function register_frontend_modfiles($sFileType="css", $bOutput=true)
        {
            $aOutput    = [];
            $head_links = '';
    // read settings from the database to enable/disable scripts
            $aFilterSettings = getOutputFilterSettings();
            $bNewFrontendFiles = (isset($aFilterSettings['RegisterModFiles']) ? $aFilterSettings['RegisterModFiles'] : false);
            switch (strtolower($sFileType)) {
                case 'css':
                    if (!defined('MOD_FRONTEND_CSS_REGISTERED')) {
                        define('MOD_FRONTEND_CSS_REGISTERED', true);
                    }
                    $aFilter = [
                      'SnippetCss',
                      'FrontendCss',
                    ];
                    $aSnippetRecords = register_addon_files($aFilter);
                    $aOutput = array_merge($aSnippetRecords['SnippetCss'], $aSnippetRecords['FrontendCss']);
                    $head_links .= prepareLink($aOutput, 'css');

                    if (!$bNewFrontendFiles){
                        $aFilter = [
                          'ScriptVars',
                          'LoadOnFly',
                          'Jquery',
                          'JqueryUI',
                        ];
                        $aSnippetRecords = register_addon_files($aFilter);

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['ScriptVars']);
                        $head_links .= prepareLink($aOutput, 'var');

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['LoadOnFly']);
                        $head_links .= prepareLink($aOutput, 'js');

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['Jquery'],$aSnippetRecords['JqueryUI']);
                        $head_links .= prepareLink($aOutput, 'js');
                            $aFilter = [
                              'SnippetJs',
                              'FrontendJs'
                            ];
                        $aSnippetRecords = register_addon_files($aFilter);

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['SnippetJs'], $aSnippetRecords['FrontendJs']);
                        $head_links .= prepareLink($aOutput, 'js');

                     }  // end not OldModFiles
// if OldStyle and jquery before body endd  load ScriptVars  LoadOnFly in head
                    if ($bNewFrontendFiles){
                        if (!defined('MOD_FRONTEND_JAVASCRIPT_REGISTERED')) {
                            define('MOD_FRONTEND_JAVASCRIPT_REGISTERED', true);
                        }
                        $aFilter = [
                          'ScriptVars',
                          'LoadOnFly',
//                          'Jquery',
//                          'JqueryUI',
                        ];
                        $aSnippetRecords = register_addon_files($aFilter);

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['ScriptVars']);
                        $head_links .= prepareLink($aOutput, 'var');

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['LoadOnFly']);
                        $head_links .= prepareLink($aOutput, 'js');
/*
                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['Jquery'],$aSnippetRecords['JqueryUI']);
                        $head_links .= prepareLink($aOutput, 'js');
*/
                     }  // end OldModFiles
                    break;
                case 'jquery':
                    if ($bNewFrontendFiles){
                        if (!defined('MOD_FRONTEND_JAVASCRIPT_REGISTERED')) {
                            define('MOD_FRONTEND_JAVASCRIPT_REGISTERED', true);
                        }
                        $aFilter = [
                          'ScriptVars',
                          'LoadOnFly',
                          'Jquery',
                          'JqueryUI',
                        ];
                        $aSnippetRecords = register_addon_files($aFilter);
                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['ScriptVars']);
                        $head_links .= prepareLink($aOutput, 'var');

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['LoadOnFly']);
                        $head_links .= prepareLink($aOutput, 'js');

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['Jquery'],$aSnippetRecords['JqueryUI']);
                        $head_links .= prepareLink($aOutput, 'js');
                    }  // end OldModFiles
                    break;
                case 'js':   // (isset($aSnippetRecords['js_head'])?$aSnippetRecords['js_head']:[])
                    if ($bNewFrontendFiles){
                        $aFilter = [
                          'SnippetJs',
                          'FrontendJs'
                        ];
                        if (!defined('MOD_FRONTEND_JAVASCRIPT_REGISTERED')) {
                            define('MOD_FRONTEND_JAVASCRIPT_REGISTERED', true);
                        }
                        $aSnippetRecords = register_addon_files($aFilter);

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['SnippetJs'], $aSnippetRecords['FrontendJs']);
                        $head_links .= prepareLink($aOutput, 'js');

                    }  // end OldModFiles
                    break;
                case 'script_old':
                    break;
                default:
                    break;  //
            }
            if ($bOutput) { print $head_links."\n";} else {return $head_links."\n";}
        }
    }

// Function to add optional module Javascript into the <body> section of the frontend
    if (!function_exists('register_frontend_modfiles_body')) {
        function register_frontend_modfiles_body($sFileType="js", $bOutput=true)
        {
            $body_links = '';
    // read settings from the database to enable/disable scripts
            $aFilterSettings = getOutputFilterSettings();
            $bNewFrontendFiles = (isset($aFilterSettings['RegisterModFiles']) ? $aFilterSettings['RegisterModFiles'] : false);
            switch (strtolower($sFileType))
            {
                case 'jquery':
                    if ($bNewFrontendFiles){
                    }
                    if (!defined('MOD_FRONTEND_BODY_JQUERY_REGISTERED')) {
                        define('MOD_FRONTEND_BODY_JQUERY_REGISTERED', true);
                    }
                        $aFilter = [
                          'Jquery',
                          'JqueryUI',
                        ];
                        $aSnippetRecords = register_addon_files($aFilter);

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['Jquery'],$aSnippetRecords['JqueryUI']);
                        $body_links .= prepareLink($aOutput, 'js');

                    break;
                case 'js':
                    if ($bNewFrontendFiles){
                    }
                    if (!defined('MOD_FRONTEND_BODY_JAVASCRIPT_REGISTERED')) {
                        define('MOD_FRONTEND_BODY_JAVASCRIPT_REGISTERED', true);
                    }
                        $aFilter = [
                          'SnippetBodyJs',
                          'FrontendBodyJs'
                        ];
                        $aSnippetRecords = register_addon_files($aFilter);

                        $aOutput = [];
                        $aOutput = array_merge($aSnippetRecords['SnippetBodyJs'], $aSnippetRecords['FrontendBodyJs']);
                        $body_links .= prepareLink($aOutput, 'js');

                    break;
                default:
                    break;
            }
        // define constant indicating that the register_frontent_files_body was invoked
            if ($bOutput) { print $body_links."\n";} else {return $body_links."\n";}
        }
    }
/* --------------------------------------------------------------------------------------*/
    function moveCssToHead($content) {
       return OutputFilterApi('CssToHead', $sContent);
    }

    if (!function_exists('xnl2br')){
        function xnl2br ($sText)
        {
        $sRetval = '';
// build array width searches
        if (!\is_array($sText)){
            $aResult = preg_split('/\R/', $sText, -1, PREG_SPLIT_DELIM_CAPTURE);
        }
        $pattern = '/[<^\/\w]+[>]|[<^\/\w]+[>].*<[\/\w>]+/is';
        foreach($aResult as $sResult) {
            $sResult .= "\n";
            if (preg_match($pattern,$sResult)){
              $sRetval .= $sResult.PHP_EOL;
              continue;
            }
            $sRetval .= preg_replace('/[\n\r]/u', '',nl2br($sResult, !defined('XHTML'))).PHP_EOL;
        }
        return $sRetval;
        }
    }

