<?php
/**
 *
 * @category        modules
 * @package         news
 * @author          WebsiteBaker Project
 * @copyright       WebsiteBaker Org. e.V.
 * @link            http://www.websitebaker.org/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.8.3
 * @requirements    PHP 5.3.6 and higher
 * @version         $Id: upgrade.php 120 2018-09-28 18:47:10Z Luisehahne $
 * @filesource      $HeadURL: svn://isteam.dynxs.de/wb/2.12.x/tags/2.12.1/modules/news/upgrade.php $
 * @lastmodified    $Date: 2018-09-28 20:47:10 +0200 (Fr, 28. Sep 2018) $
 *
 */

/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); flush(); exit; }
/* -------------------------------------------------------- */
/* **** START UPGRADE ******************************************************* */

if(!function_exists('mod_news_Upgrade'))
{
    function mod_news_Upgrade()
    {
        global $database, $MESSAGE;
        $sAddonPath = \str_replace(DIRECTORY_SEPARATOR, '/', __DIR__);
        $globalStarted = \preg_match('/upgrade\-script\.php$/', $_SERVER["SCRIPT_NAME"]);
        $sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : WB_VERSION);
        if (\version_compare($sWbVersion, '2.11.0', '<')){
            throw new \Exception ('It is not possible to upgrade from WebsiteBaker Versions before 2.11.0');
        }
// Work-out all needed path and filenames
        $sPagesPath = WB_PATH.PAGES_DIRECTORY;
        $sAccessFileRootPath = $sPagesPath;
        $sPostsPath = $sPagesPath.'/posts';

        $msg = [];
        $sTableName = TABLE_PREFIX.'mod_news_groups';
        if ($database->index_remove($sTableName,'ident_news')){;}
        $sInstallStruct = __DIR__.'/install-struct.sql';
        if (!$database->SqlImport($sInstallStruct, TABLE_PREFIX, 'upgrade' )){
            $msg[] = $database->get_error();
        }
        $aTable = array('mod_news_posts','mod_news_groups','mod_news_comments','mod_news_settings');
        for($x=0; $x<\sizeof($aTable);$x++) {
            if(($sOldType = $database->getTableEngine(TABLE_PREFIX.$aTable[$x]))) {
                if(('myisam' != \strtolower($sOldType))) {
                    if (!$database->query('ALTER TABLE `'.TABLE_PREFIX.$aTable[$x].'` Engine = \'MyISAM\' ')) {
                        $msg[] = $database->get_error();
                    }
                }
            } else {
                $msg[] = $database->get_error();
            }
        }
    // check if new fields must be added
        $doImportDate = true;
        if(!$database->field_exists(TABLE_PREFIX.'mod_news_posts', 'created_when')) {
            if(!$database->field_add(TABLE_PREFIX.'mod_news_posts', 'created_when',
                                    'INT NOT NULL DEFAULT \'0\' AFTER `commenting`')) {
                if (!$globalStarted){
                    echo $MESSAGE['RECORD_MODIFIED_FAILED'].'<br />';
                    return $msg;
                }else {
                    $admin->print_error($MESSAGE['RECORD_MODIFIED_FAILED']);
                }
            }
            if (!$globalStarted) { echo 'datafield `'.TABLE_PREFIX.'mod_news_posts`.`created_when` added.<br />'; }
        } else { $doImportDate = false; }

        if (!$database->field_exists(TABLE_PREFIX.'mod_news_posts', 'created_by')) {
            if(!$database->field_add(TABLE_PREFIX.'mod_news_posts', 'created_by',
                                    'INT NOT NULL DEFAULT \'0\' AFTER `created_when`')) {
                if (!$globalStarted){
                    echo $MESSAGE['RECORD_MODIFIED_FAILED'].'<br />';
                    return ;
                } else {
                    $admin->print_error($MESSAGE['RECORD_MODIFIED_FAILED']);
                }
            }
            if (!$globalStarted) {echo 'datafield `'.TABLE_PREFIX.'mod_news_posts`.`created_by` added.<br />'; }
        }
    // preset new fields `created_by` and `created_when` from existing values
        if($doImportDate) {
            $sql  = 'UPDATE `'.TABLE_PREFIX.'mod_news_posts` '
                  . 'SET `created_by`=`posted_by`, `created_when`=`posted_when`';
            $database->query($sql);
        }
// create /posts/ - directory if not exists
        if (!\file_exists($sPostsPath)) {
            if (\is_writable($sPagesPath)) {
                make_dir(WB_PATH.PAGES_DIRECTORY.'/posts/');
            }else {
                if(!$globalStarted){
                    $msg[] = ($MESSAGE['PAGES_CANNOT_CREATE_ACCESS_FILE']);
                }else {
                    $msg[] = $MESSAGE['PAGES_CANNOT_CREATE_ACCESS_FILE'].'<br />';
                    return $msg;
                }
            }
            if (!$globalStarted) {echo 'directory "'.PAGES_DIRECTORY.'/posts/" created.<br />'; }
        }

    // now iterate through all existing accessfiles,
    // write its creation date into database
        $oDir = new \DirectoryIterator($sPostsPath);
        $count = 0;
        foreach ($oDir as $fileinfo){
            $fileName = $fileinfo->getFilename();
            if((!$fileinfo->isDot()) &&
               ($fileName != 'index.php') &&
               (\substr_compare($fileName,PAGE_EXTENSION,(0-\strlen(PAGE_EXTENSION)),\strlen(PAGE_EXTENSION)) === 0)
              )
            {
            // save creation date from old accessfile
                if($doImportDate) {
                    $link = '/posts/'.\preg_replace('/'.\preg_quote(PAGE_EXTENSION).'$/i', '', $fileinfo->getFilename());
                    $sql  = 'UPDATE `'.TABLE_PREFIX.'mod_news_posts` SET '
                          . '`created_when`='.$fileinfo->getMTime().' '
                          . 'WHERE `link`=\''.$database->escapeString($link).'\' '
                          .   'AND `created_when`= 0';
                    $database->query($sql);
                }
            // delete old access file
                if (\is_writeable($fileinfo->getPathname())){\unlink($fileinfo->getPathname());}
                $count++;
            }
        }
        unset($oDir);
        if ($globalStarted && $count > 0) {
            $msg[] = 'save date of creation from '.$count.' old accessfiles and delete these files.<br />';
        }
// ************************************************
    // Check the validity of 'create-file-timestamp' and balance against 'posted-timestamp'
        $sql  = 'UPDATE `'.TABLE_PREFIX.'mod_news_posts` ';
        $sql .= 'SET `created_when`=`published_when` ';
        $sql .= 'WHERE `published_when`<`created_when`';
        $database->query($sql);
        $sql  = 'UPDATE `'.TABLE_PREFIX.'mod_news_posts` ';
        $sql .= 'SET `created_when`=`posted_when` ';
        $sql .= 'WHERE `published_when`=0 OR `published_when`>`posted_when`';
        $database->query($sql);
// ************************************************
    // rebuild all access-files
        $count = 0;
        $backSteps = \preg_replace('@^'.\preg_quote(WB_PATH).'@', '', $sPostsPath);
        $backSteps = \str_repeat( '../', \substr_count($backSteps, '/'));
        $sql  = 'SELECT `page_id`,`post_id`,`section_id`,`link`, `title` ';
        $sql .= 'FROM `'.TABLE_PREFIX.'mod_news_posts`';
        $sql .= 'WHERE `link` != \'\'';
        if( ($oPosts = $database->query($sql)) )
        {
            while( $aPost = $oPosts->fetchRow(MYSQLI_ASSOC) )
            {
                  $sNewLink = $aPost['link'].PAGE_EXTENSION;
                  $oAF = new \AccessFile($sAccessFileRootPath, $sNewLink, $aPost['page_id']);
                  $oAF->addVar('section_id', $aPost['section_id'], \AccessFile::VAR_INT);
                  $oAF->addVar('post_id', $aPost['post_id'], \AccessFile::VAR_INT);
                  $oAF->addVar('post_section', $aPost['section_id'], \AccessFile::VAR_INT);
                  $oAF->write();
                  unset($oAF);
                $count++;
            } // end post while
        }
        if ($globalStarted) { $msg[] = 'created '.$count.' new accessfiles.'; }

        return $msg;
    }
}

// ------------------------------------
    $callingScript = $_SERVER["SCRIPT_NAME"];
    $globalStarted = preg_match('/upgrade\-script\.php$/', $callingScript);

    $aMsg = mod_news_Upgrade();
    if (!$globalStarted && sizeof($aMsg)) {print implode("\n", $aMsg)."\n";}

/* **** END UPGRADE ********************************************************* */